#!/bin/sh
#\
exec wish "$0" "$@"

load ./libCanvasPlus.so
load ./modetest1.so

proc vset {args} {
    set cmd ".main.vscroll set $args"
    #puts $cmd
    eval $cmd
}
proc hset {args} {
    set cmd ".main.hscroll set $args"
    #puts $cmd
    eval $cmd
}
proc cxview {args} {
    set cmd ".main.canvas xview $args"
    #puts $cmd
    eval $cmd
}
proc cyview {args} {
    set cmd ".main.canvas yview $args"
    #puts $cmd
    eval $cmd
    #puts [.main.canvas yview]
}

frame .main
set C .main.canvas
set H .main.hscroll
set V .main.vscroll

canvas+ $C -width 600 -height 480 -confine 1 -yinvert 0 \
        -xscrollcommand "hset" -yscrollcommand "vset"
scrollbar $H -orient horizontal -command "cxview"
scrollbar $V -orient vertical -command "cyview"
focus $C
bind $C <KeyPress-p> {PanBinds 1 %x %y}
bind $C <KeyPress-m> {PanBinds 0 0 0}

proc PanBinds {l x y} {
    if {$l} {
	puts panzoom
	set ::prevX $x 
	set ::prevY $y
	bind $::C <Motion> {pan %x %y}
    } else {
	puts no_pan
	bind $::C <Motion> {}
    }
}

proc pan {x y} {
    set _x [expr 2*($x - $::prevX) + [winfo width $::C]/2]
    set _y [expr 2*($y - $::prevY) + [winfo height $::C]/2]
    $::C pan $_x $_y
    set ::prevX $x
    set ::prevY $y
}

grid rowconfigure .main 0 -weight 1
grid rowconfigure .main 1 -weight 0
grid columnconfigure .main 0 -weight 1
grid columnconfigure .main 1 -weight 0

grid $C -column 0 -row 0 -sticky nswe
grid $H -column 0 -row 1 -sticky we
grid $V -column 1 -row 0 -sticky ns

pack .main -expand true -fill both

menu .menu -tearoff 0
.menu add cascade -label "File" -menu .menu.file
.menu add cascade -label "Tests" -menu .menu.tests

menu .menu.file
.menu.file add command -label Exit -command exit

menu .menu.tests
.menu.tests add cascade -label "Zoom with configure" -menu .menu.tests.z1
.menu.tests add cascade -label "Zoom with command" -menu .menu.tests.z2
.menu.tests add command -label "Redraw" -command {$C redraw}
.menu.tests add separator
.menu.tests add command -label "Set mode" -command setmode
.menu.tests add command -label "Create item" -command addItem -state disabled
.menu.tests add command -label "Get binds" -command getbinds
.menu.tests add command -label "Get items" -command {puts [$C get]}
.menu.tests add command -label "Get current item" -command {puts [$C get current]}

menu .menu.tests.z1
.menu.tests.z1 add command -label "1/8" -command {$C configure -zoom 0.125}
.menu.tests.z1 add command -label "1/4" -command {$C configure -zoom 0.25}
.menu.tests.z1 add command -label "1/2" -command {$C configure -zoom 0.5}
.menu.tests.z1 add command -label "1" -command {$C configure -zoom 1}
.menu.tests.z1 add command -label "2" -command {$C configure -zoom 2}
.menu.tests.z1 add command -label "4" -command {$C configure -zoom 4}
.menu.tests.z1 add command -label "8" -command {$C configure -zoom 8}

menu .menu.tests.z2
.menu.tests.z2 add command -label "1/8" -command {$C zoom 0.125}
.menu.tests.z2 add command -label "1/4" -command {$C zoom 0.25}
.menu.tests.z2 add command -label "1/2" -command {$C zoom 0.5}
.menu.tests.z2 add command -label "1" -command {$C zoom 1}
.menu.tests.z2 add command -label "2" -command {$C zoom 2}
.menu.tests.z2 add command -label "4" -command {$C zoom 4}
.menu.tests.z2 add command -label "8" -command {$C zoom 8}

. configure -menu .menu

set moving 0 
proc move {} {
    $::I eval {
	# prevX prevY moving
	if { $moving } {
	    @move $prevX $prevY $_X_ $_Y_
	    @puts move $_I_ $prevX $prevY $_X_ $_Y_
	    set prevX $_X_
	    set prevY $_Y_
	}
    }
}
proc picOrDrop {} {
    # prevX prevY moving
    $::I eval {
	if { $moving } {
	    set moving 0
	    @puts droping
	} else {
	    set moving 1
	    set prevX $_X_
	    set prevY $_Y_
	    @puts pic $_X_ $_Y_
	}
    }
}

proc setmode {} {
    global C I
    puts "AVAILABLE MODES [$C modes]"
    set I [$C setmode modetest1]
    puts "INTERP FOR MODE IS = $I"
    $I eval {set moving 0}
    .menu.tests entryconfigure 6 -state normal
    $C bind rectangle <Enter> {puts "RECTANGLE ENTER %x %y"}
    $C bind rectangle <Leave> {puts "RECTANGLE LEAVE %x %y"}
#------------------------------------------------------
#    $C bind elipse <Enter> {puts "ELIPSE ENTER $X $Y"}
#    $C bind elipse <Leave> {puts "ELIPSE LEAVE $X $Y"}
#    $C bind elipse <Motion> {puts "ELIPSE MOTION"}
#------------------------------------------------------
    $C bind all <ButtonPress-1> picOrDrop
    $C bind all <ButtonRelease-1> picOrDrop
    $C bind all <Motion> move
#    $C bind all <Motion> {puts "ALL MOTION"}
#    $C bind all <Enter> {puts "ALL ENTER"}
#    $C bind all <Leave> {puts "ALL LEAVE"}
}

proc getbinds {} {
    global C
#    puts "ALL: [$C bind all]"
#    puts "RCT: [$C bind rectangle]"
#    puts "ELI: [$C bind elipse]"
#------------------------------------------------------
    puts [$C bind rectangle <Enter>]
    puts [$C bind rectangle <Leave>]
#    puts [$C bind rectangle <Motion>]
#------------------------------------------------------
    puts [$C bind elipse <Enter>]
    puts [$C bind elipse <Leave>]
#    puts [$C bind elipse <Motion>]
#------------------------------------------------------
#    puts [$C bind all <Motion>]
#    puts [$C bind all <Enter>]
#    puts [$C bind all <Leave>]
}

proc elabel {widget n text var} {
    label $widget.l$n -text $text
    entry $widget.e$n -textvariable $var
    grid $widget.l$n -column 0 -row $n
    grid $widget.e$n -column 1 -row $n
}

proc listBox {widget n text var args} {
    global $var
    set mb $widget.mb$n
    set mm $mb.mm
    menubutton $mb -menu $mm -textvariable $var
    menu $mm -tearoff 0
    foreach e $args {
	$mm add command -label $e -command "set $var $e"
	set $var $e
    }
    label $widget.l$n -text $text
    grid $widget.l$n -column 0 -row $n
    grid $mb -column 1 -row $n
}

proc addItem {} {
    set top .itemDialog
    catch {destroy $top}
    toplevel $top
#-----------------------------------
    frame $top.m
    listBox $top.m 0 "Item Type" T @rect @elipse
    elabel $top.m 1 "X1" X1
    elabel $top.m 2 "Y1" Y1
    elabel $top.m 3 "X2" X2
    elabel $top.m 4 "Y2" Y2
    pack $top.m
#-----------------------------------
    frame $top.b
    button $top.b.ok -text "OK" -command {
	set i [$I eval $T $X1 $Y1 $X2 $Y2]
	puts "ID $i"
	puts "bind for all         -- [$C bind all]"
	puts "bind for all <Enter> -- [$C bind all <Enter>]"
	puts "bind for all <Leave> -- [$C bind all <Leave>]"
	destroy .itemDialog
    }
    button $top.b.cancel -text "Cancel" -command {
	destroy .itemDialog
    }
    pack $top.b.ok $top.b.cancel -side left -expand true -fill x 
    pack $top.b
}


